; ----------------------------------------------------
; BAS31UT.asm      Last Update:6/4/94
;
; Version 1.0
;
; This program contains subroutines to run the R31
; analogue-to-digital converter and the clock/calander
; from BASIC when the Intel-BASIC controller is used.
;
;
; DS1215 timer chip subroutines are placed in the
; EPROM so that the DS1215 chip can be accessed
; when  the RAM is in socket U7 of the R-31 board
;
; Also included are the routines for accessing the
; clock/calendar from RROS when U7 holds the RAM chip.
; These routines are is fully compatible with the earlier
; version.  A Basic-callable version of the subroutines
; putTime and getTime, called BputTime and BgetTime are
; added to the set of subroutines so that the Intel52-
; BASIC controller may be used
;
; The program is relocatable at any 2K block boundary.
; That is, the Intel-Hex code may be placed at 0, 800h,
; 1000h, 1800h, ..., without modification.  In compliance
; with the Intel-BASIC chip, only register bank 3 is used.
;
; Copyright Rigel Corporation, 1991, 1994
;
; ====================================================
;       m a i n    p r o g r a m
; ----------------------------------------------------
; external addresses:
   tAdd    equ 0ffffh  ;  timer address (DS1215)
   BtAdd   equ 07fffh  ;  timer address
; ----------------------------------------------------

    org  04000h
    ljmp PutTByte   ; 4000h
    nop
    ljmp GetTByte   ; 4004h
    nop
    ljmp activateT  ; 4008h
    nop
    ljmp PutTime    ; 400Ch
    nop
    ljmp GetTime    ; 4010h
    nop
    ajmp BPutTime   ; 4014h -> C014h  Basic callable
    ajmp BGetTime   ; 4016h -> C016h  Basic callable
    ajmp Sconvert   ; 4018h -> C018h  Basic callable
    ajmp Dconvert   ; 401Ah -> C01Ah  Basic callable

; ====================================================
; ====================================================
; subroutine PutTByte
;
;  input    : byte in accumulator
;  output   : send byte to the DS 1215 timer chip
;  destroys : nothing - uses r0 and dptr
; ----------------------------------------------------
PutTByte:
    push  psw
    mov   psw, #0     ; select register bank 0
    push  0           ; push r0
    push  dpl
    push  dph

    mov   r0, #8      ; bit count
    mov   dptr, #tAdd ; timer address

sendbit:
    movx  @dptr, a    ; send bit
    rr    a
    djnz  r0, sendbit

    pop   dph
    pop   dpl
    pop   0
    pop   psw

    ret

; ====================================================
; subroutine GetTByte
;
;  input    : byte from the DS1215 timer chip
;  output   : byte in accumulator
;  destroys : nothing - uses b, r0, and dptr
; ----------------------------------------------------
GetTByte:
    push  psw
    mov   psw, #0    ; select register bank 0
    push  b
    push  0          ; push r0
    push  dpl
    push  dph

    mov   r0, #8      ; bit count
    mov   dptr, #tAdd ; timer address

receivebit:
    movx  a, @dptr   ; receive bit

    mov   C, acc.0   ; bit 0 to carry flag
    mov   b.0, C     ; carry flag to b.0
    mov   a, b       ; rotate b to right
    rr    a
    mov   b, a
    djnz  r0, receivebit

    mov   a, b       ; put byte in a

    pop   dph
    pop   dpl
    pop   0
    pop   b
    pop   psw

    ret

; ====================================================
; subroutine activateT
; activates the DS1215 time keeper by sending the
; 64-bit pattern
;
; first we read 16 bytes in case the timer chip
; is already activated.  note that it is not fool-
; proof to read only 8 bytes.  thus 16>>8 is read.
;
;  input    : none
;  output   : none
;  destroys : nothing - uses a, r0, and dptr
; ----------------------------------------------------
activateT:
    push  acc

    mov   r0, #128
    mov   dptr, #tAdd
at1:                ; read 16 bytes in case the
    movx  a, @dptr  ; timer chip select sequence is
    djnz   r0, at1  ; already sent

    mov   a, #0c5h
    lcall PutTByte

    mov   a, #03ah
    lcall PutTByte

    mov   a, #0a3h
    lcall PutTByte

    mov   a, #05ch
    lcall PutTByte

    mov   a, #0c5h
    lcall PutTByte

    mov   a, #03ah
    lcall PutTByte

    mov   a, #0a3h
    lcall PutTByte

    mov   a, #05ch
    lcall PutTByte

    pop   acc
    ret

; ====================================================
;  subroutine putTime
;
;  pops the 8 bytes on stack and sends them to the
;  timer chip.  the bytes, from the top of the stack
;  (last pushed), to the bottom of the stack (first
;  pushed) should be:
;
;   fraction of seconds
;   second
;   BCD (binary coded decimal) minute
;   BCD hour
;   BCD day of week
;   BCD date
;   BCD month
;   BCD year
;
;  that is, before calling putTime, push the 8 bytes;
;  first the year, then the months, and so on.
;
;  !!! this routine must be called by a lcall  !!!
;  !!! since it assumes that there are 2 bytes !!!
;  !!! of return address on top of the 8 bytes !!!
;  !!! of data on stack                        !!!
;
;  input    : 8 bytes on stack
;  output   : sends the bytes to the 8 DS1215 registers
;  destroys : a, psw, and r0, r1, and r2 of register
;             bank 0
; ----------------------------------------------------
putTime:
      mov   psw, #0
      pop   1           ; save the return address
      pop   2           ; in r1 and r2

      lcall activateT

      mov   r0, #8      ; byte count
pt1:

      pop   acc
      lcall putTbyte
      djnz  r0, pt1

      push  2           ; put the 2-byte ...
      push  1           ; ... return address
      ret

; ====================================================
;  subroutine getTime
;
;  gets the 8-byte time/date data from the timer chip
;  pushes it stack.  the bytes, from the bottom of the
;  stack (first pushed), to the top of the stack
;  (last pushed) are:
;
;   fraction of seconds
;   second
;   BCD (binary coded decimal) minute
;   BCD hour
;   BCD day of week
;   BCD date
;   BCD month
;   BCD year
;
;  that is, after getTime returns, pop the 8 bytes;
;  first the year, then the months, and so on.
;
;  !!! this routine must be called by a lcall  !!!
;  !!! since it assumes that there are 2 bytes !!!
;  !!! of return address on stack              !!!
;
;  input    : none
;  output   : 8 DS1215 registers on stack
;  destroys : a, psw, and r0, r1, and r2 of register
;             bank 0
; ----------------------------------------------------
getTime:
      mov   psw, #0
      pop   1           ; save the return address
      pop   2           ; in r1 and r2

      lcall activateT

      mov   r0, #8      ; byte count
gt1:
      lcall getTbyte
      push  acc
      djnz  r0, gt1

      push  2           ; put the 2-byte ...
      push  1           ; ... return address
      ret


; ====================================================
;
; BASIC CALLABLE VERSIONS of the
; SUBROUTINES BgetTime and BputTime
;
; Note: the support routines do not set the register
; bank.  Thus they should not be called from BASIC.
; Only BgetTime and BputTime should be called from BASIC.
;
; Only Register Bank 3 is used
;
; the information is transferred between the timer
; chip and the controller via register bank 3
;
; all calls use the ACALL instruction
;
; ====================================================


; ====================================================
; subroutine BPutTByte
;
;  input    : byte in accumulator
;  output   : send byte to the DS 1215 timer chip
;  destroys : dptr
; ----------------------------------------------------
BPutTByte:
    orl   psw, #18h    ; select register bank 3
    push  18h          ; push r0

    mov   r0, #8       ; bit count
    mov   dptr, #BtAdd ; timer address

Bsendbit:
    movx  @dptr, a     ; send bit
    rr    a
    djnz  r0, Bsendbit

    pop   18h          ; restore r0
    ret

; ====================================================
; subroutine BGetTByte
;
;  input    : byte from the DS1215 timer chip
;  output   : byte in accumulator
;  uses     : b, r0
;  destroys : dptr
; ----------------------------------------------------
BGetTByte:
    orl   psw, #18h     ; select register bank 3
    push  b
    push  18h           ; push r0

    mov   r0, #8        ; bit count
    mov   dptr, #BtAdd  ; timer address

Breceivebit:
    movx  a, @dptr      ; receive bit

    mov   C, acc.0      ; bit 0 to carry flag
    mov   b.0, C        ; carry flag to b.0
    mov   a, b          ; rotate b to right
    rr    a
    mov   b, a
    djnz  r0, Breceivebit

    mov   a, b          ; put byte in a

    pop   18h
    pop   b

    ret

; ====================================================
; subroutine BactivateT
; activates the DS1215 time keeper by sending the
; 64-bit pattern
;
; first we read 16 bytes in case the timer chip
; is already activated.  note that it is not fool-
; proof to read only 8 bytes.  thus 16>>8 is read.
;
;  input    : none
;  output   : none
;  uses     : a, r0
;  destroys : dptr
; ----------------------------------------------------
BactivateT:
    push  acc
    orl   psw, #18h  ; select register bank 3
    push  18h        ; save r0

    mov   r0, #80h
    mov   dptr, #BtAdd
Bat1:                ; read 16 bytes in case the
    movx  a, @dptr   ; timer chip select sequence is
    djnz   r0, Bat1  ; already sent

    mov   a, #0c5h
    acall BPutTByte

    mov   a, #03ah
    acall BPutTByte

    mov   a, #0a3h
    acall BPutTByte

    mov   a, #05ch
    acall BPutTByte

    mov   a, #0c5h
    acall BPutTByte

    mov   a, #03ah
    acall BPutTByte

    mov   a, #0a3h
    acall BPutTByte

    mov   a, #05ch
    acall BPutTByte

    pop   18h        ; restore r0
    pop   acc
    ret

; ====================================================
;  subroutine BputTime
;
;  Sends the 8 registers in register bank 3 to the
;  timer chip.  the register should hold the time and
;  date information as follows:
;
;  register address         value
;  -------- -------  --------------------
;     r0      18h    fraction of seconds
;     r1      19h    second
;     r2      1Ah    BCD (binary coded decimal) minute
;     r3      1Bh    BCD hour
;     r4      1Ch    BCD day of week
;     r5      1Dh    BCD date
;     r6      1Eh    BCD month
;     r7      1Fh    BCD year
;
;  input    : 8 bytes in register bank 3
;  output   : sends the bytes to the 8 DS1215 registers
; ----------------------------------------------------
BputTime:
      orl   psw, #18h   ; select register bank 3

      acall BactivateT

      mov   a, r0
      acall BputTbyte
      mov   a, r1
      acall BputTbyte
      mov   a, r2
      acall BputTbyte
      mov   a, r3
      acall BputTbyte
      mov   a, r4
      acall BputTbyte
      mov   a, r5
      acall BputTbyte
      mov   a, r6
      acall BputTbyte
      mov   a, r7
      acall BputTbyte

      anl   psw, #0e7h  ; select register bank 0
      ret

; ====================================================
;  subroutine BgetTime
;
;  gets the 8-byte time/date data from the timer chip
;  and saves them in the registers of bank 3, as
;  follows.
;
;  register address         value
;  -------- -------  --------------------
;     r0      18h    fraction of seconds
;     r1      19h    second
;     r2      1Ah    BCD (binary coded decimal) minute
;     r3      1Bh    BCD hour
;     r4      1Ch    BCD day of week
;     r5      1Dh    BCD date
;     r6      1Eh    BCD month
;     r7      1Fh    BCD year
;
;  input    : none
;  output   : 8 DS1215 registers on stack
; ----------------------------------------------------
BgetTime:
      orl   psw, #18h   ; select register bank 3

      acall BactivateT

      acall BgetTbyte
      mov   r0, a
      acall BgetTbyte
      mov   r1, a
      acall BgetTbyte
      mov   r2, a
      acall BgetTbyte
      mov   r3, a
      acall BgetTbyte
      mov   r4, a
      acall BgetTbyte
      mov   r5, a
      acall BgetTbyte
      mov   r6, a
      acall BgetTbyte
      mov   r7, a

      anl   psw, #0e7h  ; select register bank 0
      ret

; ====================================================
;
;
; ====================================================
; BASIC accessible ADC0838 analog-to-digital converter
; driver utilities
;
; Operates the ADC of the R31 board when populated
; with the 8052-BASIC chip
;
; ----------------------------------------------------
; bit addresses:
   SARS    equ P3.2    ; SAR status - not used
   CS      equ P3.3    ; chip select
   DI_DO   equ P3.4    ; data in / data out
   CLK     equ P3.5    ; clock
; ----------------------------------------------------

; ====================================================
; subroutine Sconvert
;  single-ended analog input conversion
;  the internal data register 18h (R0 of register bank 3)
;  holds the analogue channel [0..7] number.  the voltage
;  of this input with respect to the COM (the common input)
;  is converted and returned in the same register, 18h.
;
;  input    : channel # in 18h (R0 of register bank 3)
;  output   : digital value in 18h (R0 of register bank 3)
;  destroys : a
; ----------------------------------------------------
Sconvert:
    orl   psw, #18h    ; select register bank 3
    mov   a, r0        ; get channel
    anl   a, #7        ; channel must be 0..7

    setb  CS           ; deactivate ADC chip to abort
                       ; any conversion in progress
    clr   CS           ; chip select
    setb  CS           ; deactivate ADC chip to abort
                       ; any conversion in progress
    clr   CS           ; chip select

    clr   C
    acall ADCwriteBit
    acall ADCwriteBit
    setb  C
    acall ADCwriteBit  ; start bit
    acall ADCwriteBit  ; single-ended conversion

    mov   C, acc.0     ; send ODD/SIGN first
    acall ADCwriteBit  ; channel address
    mov   C, acc.2     ; SELECT BIT 1
    acall ADCwriteBit  ; channel address
    mov   C, acc.1     ; SELECT BIT 0
    acall ADCwriteBit  ; channel address

    acall ADCreadBit   ; synchronization pulse

    mov   r0, #8
Sc1:                   ; read bits, MSB first
    rl    a            ; rotate acc for next bit
    acall ADCreadBit   ; read next bit
    mov   acc.0, C     ; put in acc LSB
    djnz  r0, Sc1      ; repeat 8 times

    setb  CS           ; deactivate ADC chip
    mov   r0, a        ; put return value in 18h
    anl   psw, #0e7h   ; select register bank 0
    ret

; ====================================================
; subroutine Dconvert
;  differential analog input conversion
;  internal register 18h (R0 of register bank 3) holds
;  the analogue differential pair number [0..7],
;  as defined below:
;
;          input channels (CH0..CH7)
;          0   1   2   3   4   5   6   7
;pair #  ---------------------------------
;  0     | + | - |   |   |   |   |   |   |
;        ---------------------------------
;  1     | - | + |   |   |   |   |   |   |
;        ---------------------------------
;  2     |   |   | + | - |   |   |   |   |
;        ---------------------------------
;  3     |   |   | - | + |   |   |   |   |
;        ---------------------------------
;  4     |   |   |   |   | + | - |   |   |
;        ---------------------------------
;  5     |   |   |   |   | - | + |   |   |
;        ---------------------------------
;  6     |   |   |   |   |   |   | + | - |
;        ---------------------------------
;  7     |   |   |   |   |   |   | - | + |
;        ---------------------------------
;
;  the voltage of this input pair, provided that the + input
;  of the pair has a higher potential than the - is converted
;  and returned in the internal data register 18h, (R0 of
;  register bank 3).
;
;  input    : pair # in 18h (R0 of register bank 3)
;  output   : digital value in 18h (R0 of register bank 3)
;  destroys : a
; ----------------------------------------------------
Dconvert:
    orl   psw, #18h    ; select register bank 3
    mov   a, r0        ; get channel
    anl   a, #7        ; pair number must be 0..7

    setb  CS           ; deactivate ADC chip to abort
                       ; any conversion in progress
    clr   CS           ; chip select
    clr   C
    acall ADCwriteBit
    acall ADCwriteBit
    setb  C
    acall ADCwriteBit  ; start bit
    clr   C
    acall ADCwriteBit  ; differential input conversion

    mov   C, acc.0     ; send ODD/SIGN first
    acall ADCwriteBit  ; channel address
    mov   C, acc.2     ; SELECT BIT 1
    acall ADCwriteBit  ; channel address
    mov   C, acc.1     ; SELECT BIT 0
    acall ADCwriteBit  ; channel address

    acall ADCreadBit   ; synchronization pulse

    mov   r0, #8
Dc1:                   ; read bits, MSB first
    rl    a            ; rotate acc for next bit
    acall ADCreadBit   ; get next bit
    mov   acc.0, C     ; put in acc LSB
    djnz  r0, Dc1      ; repeat 8 times

    setb  CS           ; deactivate ADC chip
    mov   r0, a        ; put return value in 18h
    anl   psw, #0e7h   ; select register bank 0
    ret

; ====================================================
; subroutine ADCreadBit
; generates one clock pulse for the ADC.  the pulse is
; a positive going pulse 0->1->0.  The state of the
; DI_DO line is returned in the carry flag.
;  input    : none
;  output   : DI_DO in carry flag
;  destroys : nothing
; ----------------------------------------------------
ADCreadBit:
    setb  DI_DO      ; set up port for input
    clr   CLK        ; pulse clock
    setb  CLK
    mov   C, DI_DO   ; save data bit in carry flag
    clr   CLK        ; clock falls to 0
    ret

; ====================================================
; subroutine ADCwriteBit
; generates one clock pulse for the ADC.  the pulse is
; a positive going pulse 0->1->0.  The carry flag is
; copied to the DI_DO line when the pulse is high.
;  input    : DI_DO in carry flag
;  output   : none
;  destroys : nothing
; ----------------------------------------------------
ADCwriteBit:
    clr   CLK        ; set clock=0
    mov   DI_DO, C   ; put data bit in output port
    setb  CLK        ; pulse clock
    clr   CLK
    ret

; ====================================================
;
; end of program

